
%%------------------------------------------------------------------

-module(examples).
-export([fib/1]).
-export([qsort/1]).  % fun 'qsort' with 1 parm (no type, no name)
-export([by_length/1]).
-export([fact/1]).


%%------------------------------------------------------------------
%% factorial
%%------------------------------------------------------------------

fact(N) when N<0 -> err_neg_val ;
fact(0) -> 1 ;
fact(N) -> N * fact(N-1) .



%%------------------------------------------------------------------
%% tail recursive fibonacci
%%------------------------------------------------------------------

fib(0) -> 0 ; 
fib(N) when N < 0 -> err_neg_val ; 
fib(N) when N < 3 -> 1; 
fib(N) -> fib_int(N, 0, 1) .

fib_int(1, _, B) -> B ; 
fib_int(N, A, B) -> fib_int(N-1, B, A+B) .



%%------------------------------------------------------------------
%% first qsort way
%%------------------------------------------------------------------

qsort([]) -> [] ;   % If empty list, return empty (nothing to sort)
qsort([Pivot|Rest]) ->
  qsort( [Front || Front <- Rest, Front < Pivot] ) 
   ++ [Pivot]
   ++ qsort( [Back || Back <- Rest, Back >= Pivot] ) .



%%------------------------------------------------------------------
%% second qsort way
%%------------------------------------------------------------------

by_length(Lists) -> 
  qsort(Lists, fun(A,B) -> length(A) < length(B) end) .

qsort([], _ )-> [] ;  % If list is empty, return empty (ignore second param)
qsort([Pivot|Rest], Smaller) ->
   qsort([X || X <- Rest, Smaller(X,Pivot)], Smaller)
   ++ [Pivot] 
   ++ qsort([Y || Y <- Rest, not(Smaller(Y, Pivot))], Smaller) .

