
-module(tests).

-compile(export_all).

createRandomList(Nb) ->
    [random:uniform(10) || _ <- lists:seq(1, Nb)].

fastest() ->
    List = createRandomList(10000),
    StartFlatten = tools:getTimeStampMilli(),
    flatten(List),
    StopFlatten = tools:getTimeStampMilli(),
    debug:print("Flatten~n"),
    debug:print("Start at: ~p~nStop at: ~p~nTime elapsed: ~p~n", [StartFlatten, StopFlatten, (StopFlatten-StartFlatten)]),
    debug:print("#############~n~n"),
    StartReverse = tools:getTimeStampMilli(),
    reverse(List),
    StopReverse = tools:getTimeStampMilli(),
    debug:print("Reverse~n"),
    debug:print("Start at: ~p~nStop at: ~p~nTime elapsed: ~p~n", [StartReverse, StopReverse, (StopReverse-StartReverse)]),
    debug:print("#############~n~n"),
    StartMap = tools:getTimeStampMilli(),
    map(List),
    StopMap = tools:getTimeStampMilli(),
    debug:print("Map~n"),
    debug:print("Start at: ~p~nStop at: ~p~nTime elapsed: ~p~n", [StartMap, StopMap, (StopMap-StartMap)]),
    debug:print("#############~n~n").

op(A) ->
    A+2.

flatten(List) ->
    flatten(List, []).
flatten([], Accu) ->
    Accu;
flatten([Obj|Tail], Accu) ->
    flatten(Tail, lists:flatten([Accu, op(Obj)])).

reverse(List) ->
    reverse(List, []).
reverse([], Accu) ->
    lists:reverse(Accu);
reverse([Obj|Tail], Accu) ->
    reverse(Tail, [op(Obj)|Accu]).

map(List) ->
    lists:map(fun op/1, List).
